@@ -1,8 +1,9 @@
 #ifndef LINT
-static const char rcsid[] = "$Header: /proj/cvs/prod/DHCP/dst/prandom.c,v 1.1 2001/02/22 07:22:09 mellon Exp $";
+static const char rcsid[] = "$Header: /proj/cvs/prod/DHCP/dst/prandom.c,v 1.6 2007/11/30 21:51:43 fdupont Exp $";
 #endif
 /*
  * Portions Copyright (c) 1995-1998 by Trusted Information Systems, Inc.
+ * Portions Copyright (c) 2007 by Internet Systems Consortium, Inc. ("ISC")
  *
  * Permission to use, copy modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -46,8 +47,8 @@ static const char rcsid[] = "$Header: /proj/cvs/prod/DHCP/dst/prandom.c,v 1.1 20
 #endif
 
 /* 
- * the constant below is a prime number to make fixed data structues like 
- * stat and time wrap over blocks. This adds certain uncertanty to what is 
+ * the constant below is a prime number to make fixed data structures like 
+ * stat and time wrap over blocks. This adds certain randomness to what is
  * in each digested block. 
  * The prime number 2879 has the special property that when 
  * divided by 2,4 and 6 the result is also a prime numbers
@@ -58,14 +59,14 @@ static const char rcsid[] = "$Header: /proj/cvs/prod/DHCP/dst/prandom.c,v 1.1 20
 #endif
 
 /* 
- * This constant dictatates how many bits we shift to the right before using a 
+ * This constant dictates how many bits we shift to the right before using a 
  */
 #ifndef DST_SHIFT
 #define DST_SHIFT 9
 #endif
 
 /*
- * An initalizer that is as bad as any other with half the bits set 
+ * An initializer that is as bad as any other with half the bits set 
  */
 #ifndef DST_RANDOM_PATTERN
 #define DST_RANDOM_PATTERN 0x8765CA93
@@ -75,11 +76,103 @@ static const char rcsid[] = "$Header: /proj/cvs/prod/DHCP/dst/prandom.c,v 1.1 20
  */
 #define MAX_OLD 3600
 
+/*
+ *  Define a single set of configuration for prand stuff. A superset 
+ *  works okay (failed commands return no data, missing directories 
+ *  are skipped, and so on.
+ */
+static const char *cmds[] = {
+	"/usr/bin/netstat -an 2>&1",
+        "/usr/sbin/netstat -an 2>&1",
+        "/usr/etc/netstat -an 2>&1",
+	"/bin/netstat -an 2>&1",
+	"/usr/ucb/netstat -an 2>&1",
+
+	/* AIX */
+	"/bin/ps -ef 2>&1",
+	"/bin/df  2>&1",
+	"/usr/bin/uptime  2>&1",
+	"/usr/bin/printenv  2>&1",
+	"/usr/bin/netstat -s 2>&1",
+	"/usr/bin/w  2>&1",
+	/* Tru64 */
+        "/usr/bin/dig com. soa +ti=1 +retry=0 2>&1",
+	"/usr/sbin/arp -an 2>&1",
+        "/usr/ucb/uptime  2>&1",
+        "/bin/iostat  2>&1",
+	/* BSD */
+        "/bin/ps -axlw 2>&1",
+        "/usr/sbin/iostat  2>&1",
+        "/usr/sbin/vmstat  2>&1",
+	/* FreeBSD */
+        "/usr/bin/vmstat  2>&1",
+        "/usr/bin/w  2>&1",
+	/* HP/UX */
+        "/usr/bin/ps -ef 2>&1",
+	/* IRIX */
+        "/usr/etc/arp -a 2>&1",
+        "/usr/bsd/uptime  2>&1",
+        "/usr/bin/printenv  2>&1",
+        "/usr/bsd/w  2>&1",
+	/* Linux */
+        "/sbin/arp -an 2>&1",
+        "/usr/bin/vmstat  2>&1",
+	/* NetBSD */
+	/* OpenBSD */
+	/* QNX */
+	"/bin/ps -a 2>&1",
+	"/bin/sin 2>&1",
+	"/bin/sin fds 2>&1",
+	"/bin/sin memory 2>&1",
+	/* Solaris */
+        "/usr/ucb/uptime  2>&1",
+        "/usr/ucb/netstat -an 2>&1",
+
+	"/usr/bin/netstat -an 2>&1",
+        "/usr/sbin/netstat -an 2>&1",
+        "/usr/etc/netstat -an 2>&1",
+	"/bin/netstat -an 2>&1",
+	"/usr/ucb/netstat -an 2>&1",
+	NULL
+};
+
+static const char *dirs[] = {
+	"/tmp",
+	"/var/tmp",
+	".",
+	"/",
+	"/var/spool",
+	"/var/adm",
+	"/dev",
+	"/var/spool/mail",
+	"/var/mail",
+	"/home",
+	"/usr/home",
+        NULL
+};
+
+static const char *files[] = {
+        "/var/adm/messages",
+        "/var/adm/wtmp",
+        "/var/adm/lastlog",
+        "/var/log/messages",
+        "/var/log/wtmp",
+        "/var/log/lastlog",
+        "/proc/stat",
+        "/proc/rtc",
+        "/proc/meminfo",
+        "/proc/interrupts",
+        "/proc/self/status",
+        "/proc/ipstats",
+        "/proc/dumper",
+        "/proc/self/as",
+        NULL
+};
 
 /*  
  *  these two data structure are used to process input data into digests, 
  *
- *  The first structure is containts a pointer to a DST HMAC key 
+ *  The first structure contains a pointer to a DST HMAC key 
  *  the variables accompanying are used for 
  *	step : select every step byte from input data for the hash
  *	block: number of data elements going into each hash
@@ -93,7 +186,7 @@ typedef struct hash {
 } prand_hash;
 
 /*
- *  This data structure controlls number of hashes and keeps track of 
+ *  This data structure controls number of hashes and keeps track of 
  *  overall progress in generating correct number of bytes of output.
  *	output  : array to store the output data in
  *	needed  : how many bytes of output are needed
@@ -135,7 +228,7 @@ static u_int32_t ran_cnt = (DST_RANDOM_PATTERN >> 10);
 
 /* 
  * setting the quick_random generator to particular values or if both 
- * input parameters are 0 then set it to initial vlaues
+ * input parameters are 0 then set it to initial values
  */
 
 void
@@ -219,7 +312,7 @@ do_time(dst_work *work)
  *
  * do_ls() does not visit subdirectories
  * if attacker has access to machine it can guess most of the values seen
- * thus it is important to only visit directories that are freqently updated
+ * thus it is important to only visit directories that are frequently updated
  * Attacker that has access to the network can see network traffic 
  * when NFS mounted directories are accessed and know exactly the data used
  * but may not know exactly in what order data is used. 
@@ -282,7 +375,7 @@ do_ls(dst_work *work)
 		file_name[dir_len + len] = 0x0;
 		/* for all entries in dir get the stats */
 		if (stat(file_name, &buf) == 0) {
-			n++;	/* count successfull stat calls */
+			n++;	/* count successful stat calls */
 			/* copy non static fields */
 			dir_info.uid   += buf.st_uid;
 			dir_info.gid   += buf.st_gid;
@@ -335,7 +428,7 @@ unix_cmd(dst_work *work)
 		cnt += do_time(work);
 	}
 	while ((n = fread(buffer, sizeof(char), sizeof(buffer), pipe)) > 0)
-		NULL; /* drain the pipe */
+		; /* drain the pipe */
 	pclose(pipe);
 	return (cnt);		/* read how many bytes where read in */
 }
@@ -416,7 +509,7 @@ force_hash(dst_work *work, prand_hash *hash)
 
 	/* 
 	 * if more than half a block then add data to output 
-	 * otherwise adde the digest to the next hash 
+	 * otherwise add the digest to the next hash 
 	 */
 	if ((hash->digested * 2) > hash->block) {
 		i = dst_sign_data(SIG_MODE_FINAL, hash->key, &hash->ctx,
@@ -435,7 +528,7 @@ force_hash(dst_work *work, prand_hash *hash)
 /* 
  * This function takes the input data does the selection of data specified
  * by the hash control block.
- * The step varialbe in the work sturcture determines which 1/step bytes
+ * The step variable in the work structure determines which 1/step bytes
  * are used, 
  *
  */
@@ -457,7 +550,7 @@ do_hash(dst_work *work, prand_hash *hash, const u_char *input, unsigned size)
 		tmp = tp;
 		for (cnt = 0, i = hash->curr; i < size; i += hash->step, cnt++)
 			*(tp++) = input[i];
-		/* calcutate the starting point in the next input set */
+		/* calculate the starting point in the next input set */
 		hash->curr = (hash->step - (i - size)) % hash->step;
 	}
 	/* digest the data in block sizes */
@@ -511,7 +604,7 @@ my_digest(dst_work *work, const u_char *input, unsigned size)
 
 /*
  * this function gets some semi random data and sets that as an HMAC key
- * If we get a valid key this function returns that key initalized
+ * If we get a valid key this function returns that key initialized
  * otherwise it returns NULL;
  */
 static prand_hash *
@@ -529,7 +622,7 @@ get_hmac_key(int step, int block)
 	if (buff == NULL)
 		return (NULL);
 	/* do not memset the allocated memory to get random bytes there */
-	/* time of day is somewhat random  expecialy in the last bytes */
+	/* time of day is somewhat random  especially in the last bytes */
 	gettimeofday((struct timeval *) &buff[n], NULL);
 	n += sizeof(struct timeval);
 
@@ -591,7 +684,7 @@ get_hmac_key(int step, int block)
  * This function goes out and from various sources tries to generate enough
  * semi random data that a hash function can generate a random data. 
  * This function will iterate between the two main random source sources, 
- *  information from programs and directores in random order. 
+ *  information from programs and directories in random order. 
  * This function return the number of bytes added to the random output buffer. 
  */
 static unsigned
@@ -668,8 +761,8 @@ own_random(dst_work *work)
  * of randomness to the caller it will use the best available sources of 
  * randomness.
  * The current order is to use /dev/random, precalculated randomness, and 
- * finaly use some system calls and programs to generate semi random data that 
- * is then digested to generate randomness. 
+ * finally use some system calls and programs to generate semi random data
+ * that is then digested to generate randomness. 
  * This function is thread safe as each thread uses its own context, but
  * concurrent treads will affect each other as they update shared state 
  * information.
@@ -677,7 +770,7 @@ own_random(dst_work *work)
  * that is not a multiple of the output of the hash function used. 
  * 
  * If /dev/random is not available this function is not suitable to generate 
- * large ammounts of data, rather it is suitable to seed a pseudo-random 
+ * large amounts of data, rather it is suitable to seed a pseudo-random 
  * generator 
  * Returns the number of bytes put in the output buffer 
  */
@@ -779,8 +872,8 @@ dst_s_random(u_char *output, unsigned size)
  * counters that is incremented between digest operations
  * each increment operation amortizes to 2 bits changed in that value
  * for 5 counters thus the input will amortize to have 10 bits changed 
- * The counters are initaly set using the strong random function above
- * the HMAC key is selected by the same methold as the HMAC keys for the 
+ * The counters are initially set using the strong random function above
+ * the HMAC key is selected by the same method as the HMAC keys for the 
  * strong random function. 
  * Each set of counters is used for 2^25 operations 
  * 
@@ -835,13 +928,13 @@ dst_s_semi_random(u_char *output, unsigned size)
 			semi_loc += out;
 		}
 	}
-/* generate more randome stuff */
+/* generate more random stuff */
 	while (out < size) {
 		/* 
 		 * modify at least one bit by incrementing at least one counter
 		 * based on the last bit of the last counter updated update
 		 * the next one.
-		 * minimaly this  operation will modify at least 1 bit, 
+		 * minimally this  operation will modify at least 1 bit, 
 		 * amortized 2 bits
 		 */
 		for (n = 0; n < DST_NUMBER_OF_COUNTERS; n++)
